GENTITYNUM_BITS = 10
MAX_GENTITIES = 1 << GENTITYNUM_BITS
FLOAT_INT_BITS = 13
FLOAT_INT_BIAS = (1<<(FLOAT_INT_BITS-1))

## Message Types
SVC_BAD=0
SVC_NOP=1
SVC_GAMESTATE=2
SVC_CONFIGSTRING=3
SVC_BASELINE=4
SVC_SERVERCOMMAND=5
SVC_DOWNLOAD=6
SVC_SNAPSHOT=7
SVC_EOF=8

## Trajectory Types
TR_STATIONARY=0
TR_INTERPOLATE=1
TR_LINEAR=2
TR_LINEAR_STOP=3
TR_SINE=4,
TR_GRAVITY=5

## Entity Types
ET_GENERAL=0
ET_PLAYER=1
ET_ITEM=2
ET_MISSILE=3
ET_MOVER=4
ET_BEAM=5
ET_PORTAL=6
ET_SPEAKER=7
ET_PUSH_TRIGGER=8
ET_TELEPORT_TRIGGER=9
ET_INVISIBLE=10
ET_GRAPPLE=11             # grapple hooked on wall
ET_TEAM=12
ET_EVENTS=13

## Means of Death
MOD_UNKNOWN=0
MOD_SHOTGUN=1
MOD_GAUNTLET=2
MOD_MACHINEGUN=3
MOD_GRENADE=4
MOD_GRENADE_SPLASH=5
MOD_ROCKET=6
MOD_ROCKET_SPLASH=7
MOD_PLASMA=8
MOD_PLASMA_SPLASH=9
MOD_RAILGUN=10
MOD_LIGHTNING=11
MOD_BFG=12
MOD_BFG_SPLASH=13
MOD_WATER=14
MOD_SLIME=15
MOD_LAVA=16
MOD_CRUSH=17
MOD_TELEFRAG=18
MOD_FALLING=19
MOD_SUICIDE=20
MOD_TARGET_LASER=21
MOD_TRIGGER_HURT=22
MOD_GRAPPLE=23

## Team Task
TEAMTASK_NONE=0
TEAMTASK_OFFENSE=1
TEAMTASK_DEFENSE=2
TEAMTASK_PATROL=3
TEAMTASK_FOLLOW=4
TEAMTASK_RETRIEVE=5
TEAMTASK_ESCORT=6
TEAMTASK_CAMP=7

## Team
TEAM_FREE='0'
TEAM_RED='1'
TEAM_BLUE='2'
TEAM_SPECTATOR='3'
TEAM_NUM_TEAMS='4'

TEAM_STRING_MAP={
    TEAM_FREE: 'NONE',
    TEAM_RED: 'TEAM_RED',
    TEAM_BLUE: 'TEAM_BLUE',
    TEAM_SPECTATOR: 'SPECTATOR'
}

## Gametypes
GT_FFA=0                         # free for all
GT_DUEL=1          # one on one tournament
GT_RACE=2       # single player ffa
GT_TEAM=3               # team deathmatch
GT_CA=4
GT_CTF=5                 # capture the flag
GT_1FCTF=6
GT_OBELISK=7
GT_HARVESTER=8
GT_FREEZETAG=9
GT_DOMINATION=10
GT_ATTACK_AND_DEFEND=11
GT_REDROVER=12
GT_MAX_GAME_TYPE=13

GT_STRING_MAP={GT_FFA: 'FFA',
               GT_DUEL: 'DUEL',
               GT_RACE: 'RACE',
               GT_TEAM: 'TEAM',
               GT_CA: 'CA',
               GT_CTF: 'CTF',
               GT_1FCTF: '1FCTF',
               GT_OBELISK: 'OBELISK',
               GT_HARVESTER: 'HARVESTER',
               GT_FREEZETAG: 'FREEZETAG',
               GT_DOMINATION: 'DOMINATION',
               GT_ATTACK_AND_DEFEND: 'ATTACK_AND_DEFEND',
               GT_REDROVER: 'REDROVER',
}

def gametype_to_string(i):
    i=int(i)
    return GT_STRING_MAP.get(i, None)

MAX_MODELS = 256 # Not the same as Q3A
MAX_SOUNDS = 256 # Not the same as Q3A
MAX_CLIENTS = 64 
MAX_LOCATIONS = 64
MAX_PS_EVENTS = 2
MAX_STATS = 16
MAX_PERSISTANT = 16
MAX_POWERUPS = 16
MAX_WEAPONS = 16
MAX_MAP_AREA_BYTES = 32

# Configstring Definitions from Q3A source headers with mods for QL
CS_SERVERINFO = 0
CS_SYSTEMINFO = 1
CS_MUSIC = 2
CS_MESSAGE = 3               # from the map worldspawn's message field
CS_MOTD = 4              # g_motd string for server message of the day
CS_WARMUP = 5               # server time when the match will be restarted
CS_SCORES1 = 6
CS_SCORES2 = 7
CS_VOTE_TIME = 8
CS_VOTE_STRING = 9
CS_VOTE_YES = 10
CS_VOTE_NO = 11
CS_GAME_VERSION = 12
CS_LEVEL_START_TIME = 13
CS_INTERMISSION = 14
CS_ITEMS = 15
CS_BOTINFO = 16
CS_MODELS = 17

CS_SOUNDS = (CS_MODELS+MAX_MODELS)
CS_PLAYERS = (CS_SOUNDS+MAX_SOUNDS)
CS_LOCATIONS = (CS_PLAYERS+MAX_CLIENTS)
CS_LAST_GENERIC = (CS_LOCATIONS + MAX_LOCATIONS) # Should be 656

CS_FLAGSTATUS = 658
CS_SCORES1PLAYER = 659
CS_SCORES2PLAYER = 660
CS_ROUND_WARMUP = 661
CS_ROUND_START_TIME = 662 
CS_TEAMCOUNT_RED = 663
CS_TEAMCOUNT_BLUE = 664
CS_SHADERSTATE = 665
CS_NEXTMAP = 666
CS_PRACTICE = 667
CS_FREECAM = 668

CS_PAUSE_START_TIME = 669   # if this is non-zero, the game is paused
CS_PAUSE_END_TIME = 670     # 0 = pause, !0 = timeout

CS_TIMEOUTS_RED = 671        # TOs REMAINING
CS_TIMEOUTS_BLUE = 672

CS_MODEL_OVERRIDE = 673
    
CS_PLAYER_CYLINDERS = 674
CS_DEBUGFLAGS = 675
CS_ENABLEBREATH = 676
CS_TEAMLOCATORS = 677
CS_DMGTHROUGHDEPTH = 678
CS_AUTHOR = 679             # from the map worldspawn's author field
CS_AUTHOR2 = 680 
CS_ADVERT_DELAY = 681
CS_PMOVEINFO = 682
CS_ARMORINFO = 683
CS_WEAPONINFO = 684
CS_PLAYERINFO = 685
CS_1STPLAYER = 686
CS_2NDPLAYER = 687
CS_SCORE1STPLAYER = 688
CS_SCORE2NDPLAYER = 689
CS_ATMOSEFFECT =  690 # unused =  was per-map rain/snow effects
CS_MOST_DAMAGEDEALT_PLYR = 691 
CS_MOST_ACCURATE_PLYR = 692
CS_REDTEAMNAME = 693
CS_BLUETEAMNAME = 694
CS_REDTEAMCLANTAG = 695
CS_BLUETEAMCLANTAG = 696
CS_BEST_ITEMCONTROL_PLYR = 697 
CS_SERVER_OWNER = 698
CS_MOST_VALUABLE_OFFENSIVE_PLYR = 699
CS_MOST_VALUABLE_DEFENSIVE_PLYR = 700
CS_MOST_VALUABLE_PLYR = 701
CS_GENERIC_COUNT_RED = 702
CS_GENERIC_COUNT_BLUE = 703
CS_AD_SCORES = 704
CS_ROUND_WINNER = 705
CS_CUSTOM_SETTINGS = 706
CS_ROTATIONMAPS = 707
CS_ROTATIONVOTES = 708
CS_DISABLE_VOTE_UI = 709
CS_ALLREADY_TIME = 710
CS_ENEMYLOCATORS = 711
CS_INFECTED_SURVIVOR_MINSPEED = 712
CS_RACE_POINTS = 713
CS_MAX = 714

CS_STRING_MAP = {
    CS_SERVERINFO: "serverinfo",
    CS_SYSTEMINFO: "systeminfo",
    CS_MUSIC: "music",
    CS_MESSAGE: "message",
    CS_MOTD: "motd",
    CS_WARMUP: "warmup",               # server time when the match will be restarted
    CS_SCORES1: "scores1",
    CS_SCORES2: "scores2",
    CS_VOTE_TIME: "vote_time",
    CS_VOTE_STRING: "vote_string",
    CS_VOTE_YES: "vote_yes",
    CS_VOTE_NO: "vote_no",
    CS_GAME_VERSION: "game_version",
    CS_LEVEL_START_TIME: "level_start_time",
    CS_INTERMISSION: "intermission",
    CS_ITEMS: "items",
    CS_BOTINFO: "botinfo",
    CS_FLAGSTATUS: "flagstatus",
    CS_SCORES1PLAYER: "scores1player",
    CS_SCORES2PLAYER: "scores2player",
    CS_ROUND_WARMUP: "round_warmup",
    CS_ROUND_START_TIME: "round_start_time",
    CS_TEAMCOUNT_RED: "teamcount_red",
    CS_TEAMCOUNT_BLUE: "teamcount_blue",
    CS_SHADERSTATE: "shaderstate",
    CS_NEXTMAP: "nextmap",
    CS_PRACTICE: "practice",
    CS_FREECAM: "freecam",
    CS_PAUSE_START_TIME: "pause_start_time",
    CS_PAUSE_END_TIME: "pause_end_time",
    CS_TIMEOUTS_RED: "timeouts_red",
    CS_TIMEOUTS_BLUE: "timeouts_blue",
    CS_MODEL_OVERRIDE: "cs_model_override",
    CS_PLAYER_CYLINDERS: "player_cylinders",
    CS_DEBUGFLAGS: "debugflags",
    CS_ENABLEBREATH: "enablebreath",
    CS_TEAMLOCATORS: "teamlocators",
    CS_DMGTHROUGHDEPTH: "dmgthroughdepth",
    CS_AUTHOR: "author",
    CS_AUTHOR2: "author2",
    CS_ADVERT_DELAY: "advert_delauy",
    CS_PMOVEINFO: "pmoveinfo",
    CS_ARMORINFO: "armorinfo",
    CS_WEAPONINFO: "weaponinfo",
    CS_PLAYERINFO: "playerinfo",
    CS_1STPLAYER: "1stplayer",
    CS_2NDPLAYER: "2ndplayer",
    CS_SCORE1STPLAYER: "score1stplayer",
    CS_SCORE2NDPLAYER: "score2ndplayer",
    CS_ATMOSEFFECT: "atmoseffect",
    CS_MOST_DAMAGEDEALT_PLYR: "most_damagedealt_plyr",
    CS_MOST_ACCURATE_PLYR: "most_accurate_plyr",
    CS_REDTEAMNAME: "redteamname",
    CS_BLUETEAMNAME: "blueteamname",
    CS_REDTEAMCLANTAG: "redteamclantag",
    CS_BLUETEAMCLANTAG: "blueteamclantag",
    CS_BEST_ITEMCONTROL_PLYR: "best_itemcontrol_plyr",
    CS_SERVER_OWNER: "server_owner",
    CS_MOST_VALUABLE_OFFENSIVE_PLYR: "most_valuable_offensive_plyr",
    CS_MOST_VALUABLE_DEFENSIVE_PLYR: "most_valuable_deffensive_plyr",
    CS_MOST_VALUABLE_PLYR: "most_valuable_plyr",
    CS_GENERIC_COUNT_RED: "generic_count_red",
    CS_GENERIC_COUNT_BLUE: "generic_count_blue",
    CS_AD_SCORES: "cs_ad_scores",
    CS_ROUND_WINNER: "round_winner",
    CS_CUSTOM_SETTINGS: "cs_custom_settings",
    CS_ROTATIONMAPS: "rotationmaps",
    CS_ROTATIONVOTES: "rotationvotes",
    CS_DISABLE_VOTE_UI: "disable_vote_ui",
    CS_ALLREADY_TIME: "allready_time",
    CS_ENEMYLOCATORS: "enemylocators",
    CS_INFECTED_SURVIVOR_MINSPEED: "infected_survivor_minspeed",
    CS_RACE_POINTS: "race_points",
}

# WEAPONS
WP_NONE=0
WP_GAUNTLET=1
WP_MACHINEGUN=2
WP_SHOTGUN=3
WP_GRENADE_LAUNCHER=4
WP_ROCKET_LAUNCHER=5
WP_LIGHTNING=6
WP_RAILGUN=7
WP_PLASMAGUN=8
WP_BFG=9
WP_GRAPPLING_HOOK=10
WP_NAILGUN=11
WP_PROX_LAUNCHER=12
WP_CHAINGUN=13
WP_HANDS=14
WP_NUM_WEAPONS=15

WP_STRING_MAP = {
    WP_NONE: "none",
    WP_GAUNTLET: "gauntlet",
    WP_MACHINEGUN: "machinegun",
    WP_SHOTGUN: "shotgun",
    WP_GRENADE_LAUNCHER: "grenade_launcher",
    WP_ROCKET_LAUNCHER: "rocket_launcher",
    WP_LIGHTNING: "lightning",
    WP_RAILGUN: "railgun",
    WP_PLASMAGUN: "plasmagun",
    WP_BFG: "bfg",
    WP_GRAPPLING_HOOK: "grappling_hook",
    WP_HANDS: "hands",
}

userinfo_map={'c1': 'color1',
              'c2': 'color2',
              'tt': 'team_target',
              'hc': 'handicap',
              'c': 'country',
              'cn': 'clan',
              't': 'team',
              'xcn': 'extended_clan',
              'tl': 'team_leader'}




#!/bin/env python

## qldemo2json.py
## Shawn Nock, 2014

import argparse
import json
import os
import sys
import time

from qldemo import QLDemo, gametype_to_string
from qldemo.data import GameState, Snapshot
from qldemo.constants import userinfo_map, GT_TEAM, TEAM_STRING_MAP

## Configuration ##

URL_PREFIX='/demos/'
playerinfo_override = {'n': 'name',    ## The userInfo_t summary in
                       'cn': 'clan',   ## CS_PLAYER[MAX_PLAYERS] has short
                       'xcn': 'xclan', ## hard to decipher names. Any map
                       'w': 'wins',    ## here will override the name 
                       'l': 'losses',
                       't': 'team'}

### END Configuration

def main():
    parser = argparse.ArgumentParser(
        description='Summarize, in JSON a QuakeLive Demo File (dm_73)')
    parser.add_argument('file',
                        help='path of the dm_73 file to summarize')
    args = parser.parse_args()

    d = QLDemo(args.file)
    list(d)

    ## Munge playerInfo to conform to ColonelPanic's Needs
    for clientNum, player in d.gamestate.players.items():
        for key, value in player.items():
            new_name=playerinfo_override.get(key, None)
            if new_name:
                player[new_name]=player[key]
                del(player[key])
        # If it's a game where teams make sense, translate the teamId into a team name
        if int(d.gamestate.config['serverinfo']['g_gametype']) >= GT_TEAM and 'team' in player:
            player['team']=TEAM_STRING_MAP[player['team']]
        if int(d.gamestate.config['serverinfo']['g_gametype']) >= GT_TEAM and 't' in player:
            player['team']=TEAM_STRING_MAP[player['t']]

    # Calculate demo duration
    ## FIXME - I think serverTime is in msec, but it is a guess based
    ## on a skim of the Q3A source. It'd be nice to verify
    first_snap=None
    last_snap=None
    for i in d.packets:
        if i.__class__ is Snapshot:
            first_snap=i
            break
    rev = d.packets
    rev.reverse()
    for i in rev:
        if i.__class__ is Snapshot:
            last_snap=i
            break
    duration = (last_snap.serverTime - first_snap.serverTime) / 1000 if first_snap and last_snap else None
    
    output = {'filename': args.file.split(os.sep)[-1],
              'gametype': gametype_to_string(
                  d.gamestate.config['serverinfo']['g_gametype']),
              'players': d.gamestate.players,
              'size': os.stat(args.file).st_size,
              'pov': d.gamestate.clientNum,
              'timestamp': time.ctime(float(d.gamestate.config['serverinfo']['g_levelStartTime'])),
              'mapname': d.gamestate.config['serverinfo']['mapname'],
              'duration': duration}

    #output['scores'] = d.scores[-1] if d.scores else {}
    output['scores'] = {}
    if int(d.gamestate.config['serverinfo']['g_gametype']) >= GT_TEAM:
        output['scores']['TEAM_RED'] = {}
        output['scores']['TEAM_BLUE'] = {}
        output['scores']['TEAM_RED']['score'] = d.gamestate.config['scores1'].strip('"')
        output['scores']['TEAM_BLUE']['score'] = d.gamestate.config['scores2'].strip('"')
    else:
        if not d.gamestate.config['1stplayer'] in output['scores']:
            output['scores'][d.gamestate.config['1stplayer']] = {}
        if not d.gamestate.config['2ndplayer'] in output['scores']:
            output['scores'][d.gamestate.config['2ndplayer']] = {}
        output['scores'][d.gamestate.config['1stplayer']]['score'] = d.gamestate.config['scores1'].strip('"')
        output['scores'][d.gamestate.config['2ndplayer']]['score'] = d.gamestate.config['scores2'].strip('"')
        victor = d.gamestate.config['1stplayer'] if int(output['scores'][d.gamestate.config['1stplayer']]['score']) > int(output['scores'][d.gamestate.config['2ndplayer']]['score']) else None
        victor = d.gamestate.config['2ndplayer'] if int(output['scores'][d.gamestate.config['2ndplayer']]['score']) > int(output['scores'][d.gamestate.config['1stplayer']]['score']) else victor
        output['victor'] = victor

    if int(d.gamestate.config['serverinfo']['g_gametype']) >= GT_TEAM:
        output['teams'] = {}
        for team in ['TEAM_RED','TEAM_BLUE']:
            short = team.split('_')[1].lower()
            output['teams'][team] = {}
            output['teams'][team]['name'] = d.gamestate.config[short+'teamname'] if short+'teamname' in d.gamestate.config else None
            output['teams'][team]['clan'] = d.gamestate.config[short+'teamclantag'] if short+'teamclantag' in d.gamestate.config else None
            output['teams'][team]['timeouts_left'] = d.gamestate.config['timeouts_'+short] if 'timeouts_'+short in d.gamestate.config else None
            output['teams'][team]['score'] = output['scores'][team]['score']

        victor = 'TEAM_RED' if int(output['scores']['TEAM_RED']['score']) > int(output['scores']['TEAM_BLUE']['score']) else None
        victor = 'TEAM_BLUE' if int(output['scores']['TEAM_BLUE']['score']) > int(output['scores']['TEAM_RED']['score']) else victor
        output['victor'] = victor

    json.dump(output, 
              sys.stdout, 
              ensure_ascii=False,
              indent=2,
              sort_keys=True)
    return 0

if __name__ == '__main__':
    main()





#!/bin/env python

## ql-demo-parse.py
## Shawn Nock, 2014

# Standard Lib
import copy
import json
import os
import re
import struct

# C-Extension wrapping Q3A Huffman Routines
import huffman
import struct

# Constants and enum maps
from qldemo.constants import *
from qldemo.data import (GameState, EntityState, PlayerState, 
                         EntityStateNETF, PlayerStateNETF,
                         ServerCommand, Snapshot)

# Configuration

# Utility Functions

# Classes
 
class QLDemo:
    gamestate=GameState()
    packets=[]
    snapshots=[]
    scores=[]

    def __init__(self, filename):
        huffman.init()
        huffman.open(filename)

    def __iter__(self):
        while True:
            seq=huffman.readrawlong()
            length=huffman.readrawlong()
            if seq == -1 or length == -1:
                break
            huffman.fill(length)
            ack = huffman.readlong()
            cmd = huffman.readbyte()
            r = None
            if cmd == SVC_GAMESTATE: 
                r = self.parse_gamestate()
            elif cmd == SVC_SERVERCOMMAND: 
                r = self.parse_servercommand()
            elif cmd == SVC_SNAPSHOT:
                r = self.parse_snapshot()
                self.snapshots.append(r)
            self.packets.append(r)
            if r: yield r

    def parse_gamestate(self):
        ack=huffman.readlong()
        while True:
            cmd = huffman.readbyte()
            if cmd == SVC_EOF:
                break
            elif cmd == SVC_CONFIGSTRING: 
                self.parse_configstring()
            elif cmd == SVC_BASELINE:
                self.parse_baseline()
        self.gamestate.clientNum = huffman.readlong()
        self.gamestate.checksumFeed = huffman.readlong()
        return self.gamestate

    def parse_configstring(self, data=(None, None)):
        i, string = data
        if not i:
            i = huffman.readshort()
            string = huffman.readbigstring()
        dest=self.gamestate.configstrings
        fieldname=str(i)
        output=string
        if CS_STRING_MAP.get(i, None):
            dest=self.gamestate.config
            fieldname=CS_STRING_MAP.get(i)
        if string.startswith("\\"):
            output={}
            subfields = string.split('\\')
            if not fieldname in dest:
                dest[fieldname]={}
            for x in range(1, len(subfields)-1, 2):
                output[subfields[x]]=subfields[x+1]
        if i >= CS_PLAYERS and i < CS_PLAYERS+MAX_CLIENTS:
            clientNum = i-CS_PLAYERS
            fieldname=int(clientNum)
            subfields = string.split('\\')
            output = {}
            for x in range(0, len(subfields), 2):
                output[subfields[x]]=subfields[x+1]
            if output['t'] == TEAM_SPECTATOR:
                dest=self.gamestate.spectators
            else:
                dest=self.gamestate.players
        if i >= CS_SOUNDS and i < CS_SOUNDS+MAX_SOUNDS:
            dest=self.gamestate.config
            fieldname='sound'+str(i-CS_SOUNDS)
        if i >= CS_LOCATIONS and i < CS_LOCATIONS+MAX_LOCATIONS:
            dest=self.gamestate.config
            fieldname='location{:02d}'.format(i-CS_LOCATIONS)
        dest[fieldname]=output

    def parse_baseline(self):
        newnum = huffman.readbits(GENTITYNUM_BITS)
        null_state=EntityState()
        baseline = self.read_delta_entity(null_state, newnum)
        ## Broken for now, disabling for speed
        #self.gamestate.baselines[newnum]=baseline

    def read_delta_entity(self, frm, num):
        ## Check for server order to remove a baseline
        if huffman.readbits(1) == 1:
            # Don't know how we should handle this, it does mean no
            # new data; skipping for now
            return
        ## Check for 'no delta' flag
        if huffman.readbits(1) == 0:
            ## No changes, we should make 'from' a copy of
            ## 'to'... skipping for now
            return

        last_field = huffman.readbyte()
        entity = EntityState()
        netf = EntityStateNETF(entity)

        for i in range(0, last_field):
            if huffman.readbits(1) :
                if not netf.bits[i] :
                    if huffman.readbits(1) != 0:
                        if huffman.readbits(1) == 0:
                            netf.fields[i] = huffman.readbits(FLOAT_INT_BITS)
                        else :
                            netf.fields[i] = huffman.readfloat()
                else:
                    if huffman.readbits(1) != 0:
                        netf.fields[i] = huffman.readbits(netf.bits[i])

        netf.update()
        return entity

    def parse_servercommand(self):
        seq = huffman.readlong()
        string = huffman.readstring()
        
        sc=ServerCommand(seq, string)
        if sc.cmd == "scores_duel":
            sc = self.parse_duel_scores(sc)
            self.scores.append(sc.scores)
        elif sc.cmd == "scores_ctf":
            sc = self.parse_ctf_scores(sc)
            self.scores.append(sc.scores)
        elif sc.cmd == "scores":
            sc = self.parse_old_scores(sc)
            self.scores.append(sc.scores)
        elif sc.cmd == 'cs' or sc.cmd == 'bcs':
            self.update_configstring(sc)
        return sc

    def update_configstring(self, command):
        ls = command.string.split(' ')
        cs_num = int(ls[0])
        cs = ' '.join(ls[1:]).strip('"')
        self.parse_configstring((cs_num, cs))

    def parse_duel_scores(self, command):
        offset = 1
        ls = command.string.split()
        num_scores = int(ls[0])
        command.scores={}
        for client in range(num_scores):
            client_num = ls[offset+0]
            command.scores[client_num]={}
            command.scores[client_num]['score'] = ls[offset+1]
            command.scores[client_num]['ping'] = ls[offset+2]
            command.scores[client_num]['time'] = ls[offset+3]
            command.scores[client_num]['kills'] = ls[offset+4]
            command.scores[client_num]['deaths'] = ls[offset+5]
            command.scores[client_num]['accuracy'] = ls[offset+6]
            command.scores[client_num]['best_weapon'] = ls[offset+7]
            command.scores[client_num]['damage_dealt'] = ls[offset+8]
            command.scores[client_num]['impressive'] = ls[offset+9]
            command.scores[client_num]['excellent'] = ls[offset+10]
            command.scores[client_num]['gauntlet'] = ls[offset+11]
            command.scores[client_num]['perfect'] = ls[offset+12]
            command.scores[client_num]['red_armor_pickups'] = ls[offset+13]
            command.scores[client_num]['red_armor_pickup_time'] = ls[offset+14]
            command.scores[client_num]['yellow_armor_pickups'] = ls[offset+15]
            command.scores[client_num]['yellow_armor_pickup_time'] = ls[offset+16]
            command.scores[client_num]['green_armor_pickups'] = ls[offset+17]
            command.scores[client_num]['green_armor_pickup_time'] = ls[offset+18]
            command.scores[client_num]['mega_health_pickups'] = ls[offset+19]
            command.scores[client_num]['mega_healh_pickup_time'] = ls[offset+20]
            offset+=21
            command.scores[client_num]['weapon_stats'] = []
            for i in range(WP_GAUNTLET, WP_NUM_WEAPONS-1):
                weapon = {}
                weapon['hit'] = ls[offset+0]
                weapon['fired'] = ls[offset+1]
                weapon['accuracy'] = ls[offset+2]
                weapon['damage_dealt'] = ls[offset+3]
                weapon['kills'] = ls[offset+4]
                command.scores[client_num]['weapon_stats'].append(weapon)
                offset+=5
        return command

    def parse_ctf_scores(self, command):
        ls=command.string.split()
        command.scores={}
        command.scores['TEAM_RED'] = {}
        command.scores['TEAM_RED']['red_armor']    = ls[0]
        command.scores['TEAM_RED']['yellow_armor'] = ls[1]
        command.scores['TEAM_RED']['green_armor']  = ls[2]
        command.scores['TEAM_RED']['mega_health']  = ls[3]
        command.scores['TEAM_RED']['quad_damage']  = ls[4]
        command.scores['TEAM_RED']['battle_suit']  = ls[5]
        command.scores['TEAM_RED']['regeneration'] = ls[6]
        command.scores['TEAM_RED']['haste']        = ls[7]
        command.scores['TEAM_RED']['invisibility'] = ls[8]
        command.scores['TEAM_RED']['flag']         = ls[9]
        command.scores['TEAM_RED']['medkit']       = ls[10]
        command.scores['TEAM_RED']['quad_damage_time']  = ls[11]
        command.scores['TEAM_RED']['battle_suit_time']  = ls[12]
        command.scores['TEAM_RED']['regeneration_time']  = ls[13]
        command.scores['TEAM_RED']['haste_time']  = ls[14]
        command.scores['TEAM_RED']['invisibility_time']  = ls[15]
        command.scores['TEAM_RED']['flag_time']  = ls[16]
        command.scores['TEAM_BLUE'] = {}
        command.scores['TEAM_BLUE']['red_armor']    = ls[17]
        command.scores['TEAM_BLUE']['yellow_armor'] = ls[18]
        command.scores['TEAM_BLUE']['green_armor']  = ls[19]
        command.scores['TEAM_BLUE']['mega_health']  = ls[20]
        command.scores['TEAM_BLUE']['quad_damage']  = ls[21]
        command.scores['TEAM_BLUE']['battle_suit']  = ls[22]
        command.scores['TEAM_BLUE']['regeneration'] = ls[23]
        command.scores['TEAM_BLUE']['haste']        = ls[24]
        command.scores['TEAM_BLUE']['invisibility'] = ls[25]
        command.scores['TEAM_BLUE']['flag']         = ls[26]
        command.scores['TEAM_BLUE']['medkit']       = ls[27]
        command.scores['TEAM_BLUE']['quad_damage_time']  = ls[28]
        command.scores['TEAM_BLUE']['battle_suit_time']  = ls[29]
        command.scores['TEAM_BLUE']['regeneration_time']  = ls[30]
        command.scores['TEAM_BLUE']['haste_time']  = ls[31]
        command.scores['TEAM_BLUE']['invisibility_time']  = ls[32]
        command.scores['TEAM_BLUE']['flag_time']  = ls[33]

        num_scores = int(ls[34])
        command.scores['TEAM_RED']['score'] = ls[35]
        command.scores['TEAM_BLUE']['score'] = ls[36]
        offset = 0
        for client in range(num_scores):
            client_num = ls[offset+37]
            command.scores[client_num]={}
            command.scores[client_num]['team'] = ls[offset+38]
            command.scores[client_num]['premium'] = ls[offset+39]
            command.scores[client_num]['score'] = ls[offset+40]
            command.scores[client_num]['ping'] = ls[offset+41]
            command.scores[client_num]['time'] = ls[offset+42]
            command.scores[client_num]['kills'] = ls[offset+43]
            command.scores[client_num]['deaths'] = ls[offset+44]
            command.scores[client_num]['powerups'] = ls[offset+45]
            command.scores[client_num]['accuracy'] = ls[offset+46]
            command.scores[client_num]['best_weapon'] = ls[offset+47]
            command.scores[client_num]['impressive'] = ls[offset+48]
            command.scores[client_num]['excellent'] = ls[offset+49]
            command.scores[client_num]['gauntlet'] = ls[offset+50]
            command.scores[client_num]['defend'] = ls[offset+51]
            command.scores[client_num]['assist'] = ls[offset+52]
            command.scores[client_num]['captures'] = ls[offset+53]
            command.scores[client_num]['perfect'] = ls[offset+54]
            command.scores[client_num]['alive'] = ls[offset+55]
            offset+=19
        return command

    def parse_old_scores(self, command):
        ls = command.string.split()
        command.scores = {}
        num_scores = int(ls[0])
        command.scores['TEAM_RED'] = ls[1]
        command.scores['TEAM_BLUE'] = ls[2]
        offset=3
        for client in range(num_scores):
            client_num = ls[offset+0]
            command.scores[client_num]={}
            command.scores[client_num]['score'] = ls[offset+1]
            command.scores[client_num]['ping'] = ls[offset+2]
            command.scores[client_num]['time'] = ls[offset+3]
            command.scores[client_num]['powerups'] = ls[offset+4]
            command.scores[client_num]['accuracy'] = ls[offset+5]
            command.scores[client_num]['impressive'] = ls[offset+6]
            command.scores[client_num]['excellent'] = ls[offset+7]
            command.scores[client_num]['gauntlet'] = ls[offset+8]
            command.scores[client_num]['defend'] = ls[offset+9]
            command.scores[client_num]['assist'] = ls[offset+10]
            command.scores[client_num]['perfect'] = ls[offset+11]
            command.scores[client_num]['captures'] = ls[offset+12]
            command.scores[client_num]['alive'] = ls[offset+13]
            command.scores[client_num]['kills'] = ls[offset+10]
            command.scores[client_num]['deaths'] = ls[offset+11]
            command.scores[client_num]['best_weapon'] = ls[offset+12]
            offset+=18
        return command

    def parse_snapshot(self):
        new_snap = Snapshot()
        new_snap.serverTime=huffman.readlong()
        delta_num = huffman.readbyte()
        new_snap.snapFlags = huffman.readbyte()
        new_snap.areamaskLen = huffman.readbyte()
        #for i in range(new_snap.areamaskLen+1):
        #    new_snap.areamask.append(huffman.readbyte())
        #ps = self.parse_playerstate()
        #new_snap.playerstate=ps
        return new_snap

    def parse_playerstate(self):
        last_field=huffman.readbyte()
        player=PlayerState()
        netf=PlayerStateNETF(player)

        playerStateFieldsNum  = len( netf.bits )

        if last_field > playerStateFieldsNum :
            return None

        for i in range( 0, last_field) :
            if huffman.readbits( 1 ) :
                if netf.bits[ i ] == 0 :
                    if huffman.readbits( 1 ) == 0 :
                        netf.fields[ i ] = huffman.readbits( FLOAT_INT_BITS ) - FLOAT_INT_BIAS
                    else :
                        netf.fields[ i ] = huffman.readfloat()
                else :
                    bits = netf.bits[ i ]
                    netf.fields[ i ] = huffman.readbits( bits )
        netf.update()

        if huffman.readbits( 1 ) :
            if huffman.readbits( 1 ) :
                c = huffman.readshort()
                for i in range( MAX_STATS ) :
                    if c & ( 1 << i ) :
                        player.stats[ i ] = huffman.readshort()

            if huffman.readbits( 1 ) :
                c = huffman.readshort()
                for i in range( MAX_PERSISTANT ) :
                    if c & ( 1 << i ) :
                        player.persistant[ i ] = huffman.readshort()

            if huffman.readbits( 1 ) :
                c = huffman.readshort()
                for i in range( MAX_WEAPONS ) :
                    if c & ( 1 << i ) :
                        player.ammo[ i ] = huffman.readshort()

            if huffman.readbits( 1 ) :
                c = huffman.readshort()
                for i in range( MAX_POWERUPS ) :
                    if c & ( 1 << i ) :
                        player.powerups[ i ] = huffman.readlong()

        return player




from qldemo.constants import *

class FlattenableObject:
    def flatten(self):
        out = {}
        for key, value in self.__dict__.items():
            if key.startswith('__'):
                continue
            elif hasattr(value, 'flatten'):
                out[key]=value.flatten()
                continue
            if value.__class__ is list:
                out[key]=[member.flatten() for member in value if hasattr(member, 'flatten')]
                continue
            if value.__class__ is dict:
                out2={}
                for k, v in value.items():
                    if hasattr(v, 'flatten'):
                        out2[k]=v.flatten()
                    else:
                        out2[k]=v
                out[key]=out2
                continue
            out[key]=value
        return out

#class Scores(FlattenableObject):
#    def __init__(self, score_string):
#        

class GameState(FlattenableObject):
    def __init__(self):
        self.configstrings = {}
        self.config = {}
        self.players = {}
        self.spectators = {}
        self.baselines = {}
        self.scores = {}

class Trajectory(FlattenableObject):
  def __init__(self):
    self.trType = 0
    self.trTime = 0
    self.rDuration = 0
    self.trBase = [ 0 ] * 3
    self.trDelta = [ 0 ] * 3
    self.gravity = 0

class ServerCommand(FlattenableObject):
    seq = None
    cmd = None
    string = None

    def __init__(self, seq, string):
        self.seq = seq
        self.cmd = string.split()[0]
        self.string = ' '.join(string.split()[1:])

class PlayerState(FlattenableObject):
  def __init__(self):
    self.commandTime = 0
    self.pm_type = 0
    self.bobCycle = 0
    self.pm_flags = 0
    self.pm_time = 0
    self.origin = [ 0 ] * 3
    self.velocity = [ 0 ] * 3
    self.weaponTime = 0
    self.gravity = 0
    self.speed = 0
    self.delta_angles = [ 0 ] * 3
    self.groundEntityNum = 0
    self.legsTimer = 0
    self.legsAnim = 0
    self.torsoTimer = 0
    self.torsoAnim = 0
    self.movementDir = 0
    self.grapplePoint = [ 0 ] * 3
    self.eFlags = 0
    self.eventSequence = 0
    self.events = [ 0 ] * MAX_PS_EVENTS
    self.eventParms = [ 0 ] * MAX_PS_EVENTS
    self.externalEvent = 0
    self.externalEventParm = 0
    self.externalEventTime = 0
    self.clientNum = 0
    self.weapon = 0
    self.weaponstate = 0
    self.viewangles = [ 0 ] * 3
    self.viewheight = 0
    self.damageEvent = 0
    self.damageYaw = 0
    self.damagePitch = 0
    self.damageCount = 0
    self.stats = [ 0 ] * MAX_STATS
    self.persistant = [ 0 ] * MAX_PERSISTANT
    self.powerups = [ 0 ] * MAX_POWERUPS
    self.ammo = [ 0 ] * MAX_WEAPONS
    self.generic1 = 0
    self.loopSound = 0
    self.jumppad_ent = 0
    self.ping = 0
    self.pmove_framecount = 0
    self.jumppad_frame = 0
    self.entityEventSequence = 0

class PlayerStateNETF:
  def __init__( self, player ):
    self.player = player
    self.fields = [ 0 ] * 48
    self.bits = []
    self.bits.append( 32 )
    self.bits.append( 0 )
    self.bits.append( 0 )
    self.bits.append( 8 )
    self.bits.append( 0 )
    self.bits.append( 0 )
    self.bits.append( 0 )
    self.bits.append( 0 )
    self.bits.append( -16 )
    self.bits.append( 0 )
    self.bits.append( 0 )
    self.bits.append( 8 )
    self.bits.append( -16 )
    self.bits.append( 16 )
    self.bits.append( 8 )
    self.bits.append( 4 )
    self.bits.append( 8 )
    self.bits.append( 8 )
    self.bits.append( 8 )
    self.bits.append( 16 )
    self.bits.append( GENTITYNUM_BITS )
    self.bits.append( 4 )
    self.bits.append( 16 )
    self.bits.append( 10 )
    self.bits.append( 16 )
    self.bits.append( 16 )
    self.bits.append( 16 )
    self.bits.append( 8 )
    self.bits.append( -8 )
    self.bits.append( 8 )
    self.bits.append( 8 )
    self.bits.append( 8 )
    self.bits.append( 8 )
    self.bits.append( 8 )
    self.bits.append( 8 )
    self.bits.append( 16 )
    self.bits.append( 16 )
    self.bits.append( 12 )
    self.bits.append( 8 )
    self.bits.append( 8 )
    self.bits.append( 8 )
    self.bits.append( 5 )
    self.bits.append( 0 )
    self.bits.append( 0 )
    self.bits.append( 0 )
    self.bits.append( 0 )
    self.bits.append( 10 )
    self.bits.append( 16 )


    assert len( self.bits ) == len( self.fields )

  def __setitem__(self,index,value):
    self.fields[ index ] = value

  def __getitem__(self,index):
    return self.fields[ index ]
    
  def update(self):
    self.player.commandTime = self.fields[ 0 ]
    self.player.origin[ 0 ] = self.fields[ 1 ]
    self.player.origin[ 1 ] = self.fields[ 2 ]
    self.player.bobCycle = self.fields[ 3 ]
    self.player.velocity[ 0 ] = self.fields[ 4 ]
    self.player.velocity[ 1 ] = self.fields[ 5 ]
    self.player.viewangles[ 1 ] = self.fields[ 6 ]
    self.player.viewangles[ 0 ] = self.fields[ 7 ]
    self.player.weaponTime = self.fields[ 8 ]
    self.player.origin[ 2 ] = self.fields[ 9 ]
    self.player.velocity[ 2 ] = self.fields[ 10 ]
    self.player.legsTimer = self.fields[ 11 ]
    self.player.pm_time = self.fields[ 12 ]
    self.player.eventSequence = self.fields[ 13 ]
    self.player.torsoAnim = self.fields[ 14 ]
    self.player.movementDir = self.fields[ 15 ]
    self.player.events[ 0 ] = self.fields[ 16 ]
    self.player.legsAnim = self.fields[ 17 ]
    self.player.events[ 1 ] = self.fields[ 18 ]
    self.player.pm_flags = self.fields[ 19 ]
    self.player.groundEntityNum = self.fields[ 20 ]
    self.player.weaponstate = self.fields[ 21 ]
    self.player.eFlags = self.fields[ 22 ]
    self.player.externalEvent = self.fields[ 23 ]
    self.player.gravity = self.fields[ 24 ]
    self.player.speed = self.fields[ 25 ]
    self.player.delta_angles[ 1 ] = self.fields[ 26 ]
    self.player.externalEventParm = self.fields[ 27 ]
    self.player.viewheight = self.fields[ 28 ]
    self.player.damageEvent = self.fields[ 29 ]
    self.player.damageYaw = self.fields[ 30 ]
    self.player.damagePitch = self.fields[ 31 ]
    self.player.damageCount = self.fields[ 32 ]
    self.player.generic1 = self.fields[ 33 ]
    self.player.pm_type = self.fields[ 34 ]
    self.player.delta_angles[ 0 ] = self.fields[ 35 ]
    self.player.delta_angles[ 2 ] = self.fields[ 36 ]
    self.player.torsoTimer = self.fields[ 37 ]
    self.player.eventParms[ 0 ] = self.fields[ 38 ]
    self.player.eventParms[ 1 ] = self.fields[ 39 ]
    self.player.clientNum = self.fields[ 40 ]
    self.player.weapon = self.fields[ 41 ]
    self.player.viewangles[ 2 ] = self.fields[ 42 ]
    self.player.grapplePoint[ 0 ] = self.fields[ 43 ]
    self.player.grapplePoint[ 1 ] = self.fields[ 44 ]
    self.player.grapplePoint[ 2 ] = self.fields[ 45 ]
    self.player.jumppad_ent = self.fields[ 46 ]
    self.player.loopSound = self.fields[ 47 ]

class Snapshot(FlattenableObject):
  def __init__(self):
    self.valid = False
    self.snapFlags = 0
    self.serverTime = 0
    self.messageNum = 0
    self.deltaNum = 0
    self.ping = 0
    self.areamask = [ 0 ] * MAX_MAP_AREA_BYTES
    self.cmdNum = 0
    self.ps = None
    self.numEntities = 0
    self.parseEntitiesNum = 0
    self.serverCommandNum = 0

class EntityState(FlattenableObject):
    def __init__(self):
        self.number = 0
        self.eType = 0
        self.eFlags = 0
        self.pos = Trajectory()
        self.apos = Trajectory()
        self.time = 0
        self.time2 = 0
        self.origin = [ 0 ] * 3
        self.origin2 = [ 0 ] * 3
        self.angles = [ 0 ] * 3
        self.angles2 = [ 0 ] * 3
        self.otherEntityNum = 0
        self.otherEntityNum2 = 0
        self.groundEntityNum = 0
        self.constantLight = 0
        self.loopSound = 0
        self.modelindex = 0
        self.modelindex2 = 0
        self.clientNum = 0
        self.frame = 0
        self.solid = 0
        self.event = 0
        self.eventParm = 0
        self.powerups = 0
        self.weapon = 0
        self.legsAnim = 0
        self.torsoAnim = 0
        self.generic1 = 0

  

class EntityStateNETF:
  def __init__(self,entity):
    self.entity = entity
    self.fields = [ 0 ] * 53
    self.bits = []
    self.bits.append( 32 )
    self.bits.append( 0 )
    self.bits.append( 0 )
    self.bits.append( 0 )
    self.bits.append( 0 )
    self.bits.append( 0 )
    self.bits.append( 0 )
    self.bits.append( 0 )
    self.bits.append( 0 )
    self.bits.append( 32 )
    self.bits.append( 10 )
    self.bits.append( 0 )
    self.bits.append( 8 )
    self.bits.append( 8 )
    self.bits.append( 8 )
    self.bits.append( 8 )
    self.bits.append( GENTITYNUM_BITS )
    self.bits.append( 8 )
    self.bits.append( 19 )
    self.bits.append( GENTITYNUM_BITS )
    self.bits.append( 8 )
    self.bits.append( 8 )
    self.bits.append( 0 )
    self.bits.append( 32 )
    self.bits.append( 8 )
    self.bits.append( 0 )
    self.bits.append( 0 )
    self.bits.append( 0 )
    self.bits.append( 24 )
    self.bits.append( 16 )
    self.bits.append( 8 )
    self.bits.append( GENTITYNUM_BITS )
    self.bits.append( 8 )
    self.bits.append( 8 )
    self.bits.append( 0 )
    self.bits.append( 0 )
    self.bits.append( 0 )
    self.bits.append( 8 )
    self.bits.append( 0 )
    self.bits.append( 32 )
    self.bits.append( 32 )
    self.bits.append( 32 )
    self.bits.append( 0 )
    self.bits.append( 0 )
    self.bits.append( 0 )
    self.bits.append( 0 )
    self.bits.append( 32 )
    self.bits.append( 32 )
    self.bits.append( 0 )
    self.bits.append( 0 )
    self.bits.append( 0 )
    self.bits.append( 32 )
    self.bits.append( 16 )

    assert len( self.bits ) == len( self.fields )

  def __setitem__(self,index,value):
    self.fields[ index ] = value

  def update(self):
    self.entity.pos.trTime = self.fields[ 0 ]
    self.entity.pos.trBase[ 0 ] = self.fields[ 1 ]
    self.entity.pos.trBase[ 1 ] = self.fields[ 2 ]
    self.entity.pos.trDelta[ 0 ] = self.fields[ 3 ]
    self.entity.pos.trDelta[ 1 ] = self.fields[ 4 ]
    self.entity.pos.trBase[ 2 ] = self.fields[ 5 ]
    self.entity.apos.trBase[ 1 ] = self.fields[ 6 ]
    self.entity.pos.trDelta[ 2 ] = self.fields[ 7 ]
    self.entity.apos.trBase[ 0 ] = self.fields[ 8 ]
    self.entity.apos.gravity = self.fields[ 9 ]
    self.entity.event = self.fields[ 10 ]
    self.entity.angles2[ 1 ] = self.fields[ 11 ]
    self.entity.eType = self.fields[ 12 ]
    self.entity.torsoAnim = self.fields[ 13 ]
    self.entity.eventParm = self.fields[ 14 ]
    self.entity.legsAnim = self.fields[ 15 ]
    self.entity.groundEntityNum = self.fields[ 16 ]
    self.entity.pos.trType = self.fields[ 17 ]
    self.entity.eFlags = self.fields[ 18 ]
    self.entity.otherEntityNum = self.fields[ 19 ]
    self.entity.weapon = self.fields[ 20 ]
    self.entity.clientNum = self.fields[ 21 ]
    self.entity.angles[ 1 ] = self.fields[ 22 ]
    self.entity.pos.trDuration = self.fields[ 23 ]
    self.entity.apos.trType = self.fields[ 24 ]
    self.entity.origin[ 0 ] = self.fields[ 25 ]
    self.entity.origin[ 1 ] = self.fields[ 26 ]
    self.entity.origin[ 2 ] = self.fields[ 27 ]
    self.entity.solid = self.fields[ 28 ]
    self.entity.powerups = self.fields[ 29 ]
    self.entity.modelindex = self.fields[ 30 ]
    self.entity.otherEntityNum2 = self.fields[ 31 ]
    self.entity.loopSound = self.fields[ 32 ]
    self.entity.generic1 = self.fields[ 33 ]
    self.entity.origin2[ 2 ] = self.fields[ 34 ]
    self.entity.origin2[ 0 ] = self.fields[ 35 ]
    self.entity.origin2[ 1 ] = self.fields[ 36 ]
    self.entity.modelindex2 = self.fields[ 37 ]
    self.entity.angles[ 0 ] = self.fields[ 38 ]
    self.entity.time = self.fields[ 39 ]
    self.entity.apos.trTime = self.fields[ 40 ]
    self.entity.apos.trDuration = self.fields[ 41 ]
    self.entity.apos.trBase[ 2 ] = self.fields[ 42 ]
    self.entity.apos.trDelta[ 0 ] = self.fields[ 43 ]
    self.entity.apos.trDelta[ 1 ] = self.fields[ 44 ]
    self.entity.apos.trDelta[ 2 ] = self.fields[ 45 ]
    self.entity.apos.gravity = self.fields[ 46 ]
    self.entity.time2 = self.fields[ 47 ]
    self.entity.angles[ 2 ] = self.fields[ 48 ]
    self.entity.angles2[ 0 ] = self.fields[ 49 ]
    self.entity.angles2[ 2 ] = self.fields[ 50 ]
    self.entity.constantLight = self.fields[ 51 ]
    self.entity.frame = self.fields[ 52 ]



import ez_setup
ez_setup.use_setuptools()
from setuptools import setup, Extension, find_packages

m = Extension('huffman',
              sources = ['huffman/huffman.c','huffman/pyhuffman.c'])

setup (name = 'ql-demo',
       version = '0.4.6',
       ext_modules = [m],
       packages = find_packages(),
       scripts = ['qldemo2json.py','ez_setup.py','qldemosummary.py'],
       author = "Shawn Nock",
       author_email = "nock@nocko.se",
       description = "Wrapper for Q3A Huffman Code Routines and QuakeLive Demo Utility Classes",
       license = "GPLv3",
       keywords = "quake quakelive demo dm_73 quakecon",
       url = "https://aphr.asia/gitweb/?p=qldemo-python.git;a=summary",
       zip_safe = True
)





#!/bin/env python

## qldemo2json.py
## Shawn Nock, 2014

DEMO_PATH='demos/'
OUTPUT_PATH='api/'

import argparse
import os
import json
import sys

from qldemo import QLDemo

def main():
    parser = argparse.ArgumentParser(
        description='Summarize, in JSON a QuakeLive Demo File (dm_73)')
    parser.add_argument('file',
                        help='path of the dm_73 file to summarize')
    args = parser.parse_args()

    d = QLDemo(args.file)
    list(d)
    json.dump([packet.flatten() for packet in d.packets], 
              sys.stdout, sort_keys=True, ensure_ascii=False, indent=2)
    #json.dump(d.scores, sys.stdout, sort_keys=True, ensure_ascii=False, indent=2)

if __name__ == '__main__':
    main()


